package CatalystX::state51::ControllerRole::BaseAndOrgIdentifer;
use MooseX::MethodAttributes::Role;
use MooseX::Types::Moose qw/ Bool /;
use MooseX::Types::Common::String qw/ NonEmptySimpleStr /;
use MooseX::Types::LoadableClass qw/ LoadableClass /;
use Moose::Autobox;
use List::MoreUtils qw/ any /;
use Try::Tiny;
use namespace::autoclean;

has _org_class => (
    init_arg => 'org_class',
    isa => LoadableClass,
    coerce => 1,
    default => 'state51::Organisation',
    handles => {
        _fetch_org => 'fetch_one',
    },
);

has _org_stash_key => (
    init_arg => 'org_stash_key',
    isa => NonEmptySimpleStr,
    is => 'ro',
    default => 'organisation',
);

has all_orgs_allowed => (
    isa => Bool,
    is => 'ro',
    default => sub { 0 },
);

sub base : Chained('/base') PathPart('changeme') CaptureArgs(0) {}

sub org_identifier : Chained('base') PathPart('') CaptureArgs(1) {
    my ($self, $c, $catalogue_identifier) = @_;
    $c->detach('/error404') unless is_NonEmptySimpleStr($catalogue_identifier);
    $catalogue_identifier =~ s/^~//;
    my $org;

    if (lc($catalogue_identifier) eq "all" && $self->all_orgs_allowed) {
        $org = $c->stash->{role}->organisation_reps_and_licensed_orgs;
    }
    else {
        $org = try { $self->_fetch_org(identifier => $catalogue_identifier) };
        $c->detach('/error404') unless $org;
    }
    $c->stash( organisation_identifier => $catalogue_identifier );
    $c->stash( $self->_org_stash_key() => $org );
}

sub own_org : Chained('org_identifier') PathPart('') CaptureArgs(0) {
    my ($self, $c) = @_;
    $c->detach('/error403') unless $c->stash->{role};
    my $org = $c->stash->{organisation};
    $c->detach('/error404') unless (
        any { $_->id == $org->id } $c->stash->{role}->organisation_reps->flatten
        or $c->stash->{is_user_admin}
    );
}

1;

