package CatalystX::state51::ControllerRole::ImproveThisPage;
use MooseX::MethodAttributes::Role;
use Moose::Autobox;
use Try::Tiny;
use MIME::Base64 qw(decode_base64);
use Encode qw(decode);
use JSON;
use namespace::autoclean;

has improve_this_page_model => (
    is => 'ro',
    default => 'Ticket',
);

sub improve_this_page : Chained('base') PathPart('improve_this_page')  Args(0) ActionClass('REST') {}

sub improve_this_page_POST {
    my ($self, $c) = @_;
    my $result = {};
    my $params = $c->req->parameters;

    my $requestor = ($c->stash->{role})? $c->stash->{role}->email : undef;
    $requestor = $params->{email} if (!$requestor && $params->{email});
    unless ($requestor) {
        $c->res->body(encode_json({error => 'No email provided'}));
        $c->detach;
    }

    my $feedback = $params->{feedback};
    unless ($feedback) {
        $c->res->body(encode_json({error => 'No feedback provided'}));
        $c->detach;
    }

    my $page_details = decode_json(decode("UTF-8",decode_base64($params->{page_details})));
    my $appname = ref($c) || $c;
    my $versionstr = ref($c) . '::VERSION';
    my $version = do { no strict 'refs'; $$versionstr };
    my $subject = sprintf('[%s] Improve This Page: %s (%s)', $appname, $page_details->{action}, $page_details->{method}) ;
    my $message = sprintf(
        "Feedback for page %s (%s):\n\n%s\n\n-----\nPage Details:\n\nAction: %s\nWeb Code Version: %s\nPlatform Version: %s\nReferer: %s\nParameters: %s",
        $page_details->{uri},
        $page_details->{method},
        $feedback,
        $page_details->{action}, $version, $state51::Base::VERSION,
        ($page_details->{referer})?$page_details->{referer}:'- none -',
        (scalar keys %{$page_details->{parameters}})?JSON->new->utf8(1)->pretty(1)->indent(1)->encode($page_details->{parameters}):'- none -',
    );

    my $model = try { $c->model($self->improve_this_page_model) };
    if (!$model) {
        $c->log->warn(sprintf("Cannot find the '%s' model for %s", $self->improve_this_page_model, ref($self))) if $c->debug;
        $c->res->body(encode_json({error => 'Feedback cannot be sent, please try again later'}));
        $c->detach;
    }
    if (!$model->can('raise')) {
        $c->log->warn(sprintf("Your '%s' model used by '%s' does not have a 'raise' method", $self->improve_this_page_model, ref($self))) if $c->debug;
        $c->res->body(encode_json({error => 'Feedback cannot be sent, please try again later'}));
        $c->detach;
    }

    try {
        $model->raise($subject, $message, $requestor);
        $result->{notice} = 'Thanks for your feedback. You will receive a confirmation email shortly';
    }
    catch {
        $c->log->warn("Error sending support request:" . $_);
        $result->{error} = 'Feedback request couldn\'t be sent due an error on the system, please try again later';
    };
    $c->res->header('Content-Type' => 'application/json');
    $c->res->body(encode_json($result));
}
1;

