package CatalystX::state51::Ticket;
use state51::Moose;
use MooseX::Types::Moose qw/Int Str/;
use Try::Tiny;
use state51::Mail::Send;
use RT::Client::REST;
use RT::Client::REST::Ticket;
use Encode qw/ encode /;
use Data::Dumper;
use Carp qw/confess/;
use namespace::autoclean;

has username => (
    is => 'ro',
    isa => Str,
    default => 'platform',
);

has password => (
    is => 'ro',
    isa => Str,
    default => 'cifVukghib',
);

has server => (
    is => 'ro',
    isa => Str,
    default => 'http://rt.ci-support.com',
);

has queue => (
    is => 'ro',
    isa => Str,
    default => 'support',
);

has timeout => (
    is => 'ro',
    isa => Int,
    default => 10,
);

has _rt => (
    isa => 'RT::Client::REST',
    is => 'ro',
    default => sub {
        my $self = shift;
        my $server = $self->server || 'http://rt.ci-support.com';
        my ($i, $rt, $error) = (0, undef, undef);
        while (!$rt && $i++ < 2) {
            try {
                $rt = RT::Client::REST->new(
                    server => $server,
                    timeout => $self->timeout,
                );
                $rt->login(username => $self->username, password => $self->password);
            }
            catch {
                undef $rt;
                $error = $_;
            };
        }
        confess("Could not login to RT at $server with '" . $self->username . "': " . $error) unless $rt;
        return $rt;
    },
    lazy => 1,
);

has _rt_ticket => (
    isa => 'RT::Client::REST::Ticket',
    is => 'ro',
    writer => '_set_rt_ticket',
    handles => {
        get_ticket_status => 'status',
    },
);

method raise($subject, $message, $requestor, $queue) {
    return if $ENV{HARNESS_ACTIVE};
    confess ('Subject, Message and Requestor are required') unless ($subject && $message && $requestor);
    $message = encode('utf-8',$message."\n\n--\nMessage sent via CI Support website\n");
    $queue ||= $self->queue;
    try {
        my $ticket = RT::Client::REST::Ticket->new(
            rt => $self->_rt,
            queue => $queue,
            subject => $subject,
            creator => q{CI Support <cisupport@ci-support.com>},
            requestors => [$requestor],
            cc => [$requestor],
        )->store(text => $message);
        $self->_set_rt_ticket($ticket);
        try {
            $ticket->cc([]);
            $ticket->store;
        }
        catch {
            warn("Couldn't delete requestor from CC for ticket '.$ticket->id.'. Error: ". $_);
        };
    }
    catch {
        my $mail = state51::Mail::Send->new;
        $mail->mail({
            to => $queue.'@ci-support.com',
            from => $requestor,
            subject => $subject,
            raw_output => $message,
            precedence => 'normal',
        });
    };
}

__PACKAGE__->meta->make_immutable;
1;

